@@ -58,7 +58,7 @@ static int get_match(const char *input, const char *pattern, char **match)
     if (ret)
     {
         regerror(ret, &re, err, sizeof(err));
-        fprintf(stderr, "regex error: %s", err);
+        tio_error_printf("Regex failure: %s", err);
         return ret;
     }
 
@@ -124,9 +124,13 @@ static int data_handler(void *user, const char *section, const char *name,
         {
             option.output_delay = atoi(value);
         }
-        else if (!strcmp(name, "dtr-pulse-duration"))
+        else if (!strcmp(name, "output-line-delay"))
         {
-            option.dtr_pulse_duration = atoi(value);
+            option.output_line_delay = atoi(value);
+        }
+        else if (!strcmp(name, "line-pulse-duration"))
+        {
+            line_pulse_duration_option_parse(value);
         }
         else if (!strcmp(name, "no-autoconnect"))
         {
@@ -215,12 +219,16 @@ static int data_handler(void *user, const char *section, const char *name,
                 // Ignore
                 return 0;
             }
-
-            if (!strcmp(value, "none"))
+            else if (!strcmp(value, "none"))
             {
                 option.color = -1; // No color
                 return 0;
             }
+            else if (!strcmp(value, "bold"))
+            {
+                option.color = 256; // Bold
+                return 0;
+            }
 
             option.color = atoi(value);
             if ((option.color < 0) || (option.color > 255))
@@ -233,6 +241,15 @@ static int data_handler(void *user, const char *section, const char *name,
             asprintf(&c->socket, "%s", value);
             option.socket = c->socket;
         }
+        else if (!strcmp(name, "prefix-ctrl-key"))
+        {
+            if (ctrl_key_code(value[0]) > 0)
+            {
+                option.prefix_code = ctrl_key_code(value[0]);
+                option.prefix_key = value[0];
+            }
+        }
+
     }
     return 0;
 }
@@ -326,7 +343,7 @@ void config_file_parse(void)
     c = malloc(sizeof(struct config_t));
     if (!c)
     {
-        fprintf(stderr, "Error: Insufficient memory allocation");
+        tio_error_printf("Insufficient memory allocation");
         exit(EXIT_FAILURE);
     } 
     memset(c, 0, sizeof(struct config_t));
@@ -351,7 +368,7 @@ void config_file_parse(void)
     ret = ini_parse(c->path, data_handler, NULL);
     if (ret < 0)
     {
-        fprintf(stderr, "Error: Unable to parse configuration file (%d)", ret);
+        tio_error_printf("Unable to parse configuration file (%d)", ret);
         exit(EXIT_FAILURE);
     }
     free(c->section_name);
@@ -364,7 +381,7 @@ void config_file_parse(void)
         ret = ini_parse(c->path, section_name_search_handler, NULL);
         if (!c->section_name)
         {
-            debug_printf("Unable to match user input to configuration section (%d)", ret);
+            tio_debug_printf("Unable to match user input to configuration section (%d)", ret);
             return;
         }
     }
@@ -373,7 +390,7 @@ void config_file_parse(void)
     ret = ini_parse(c->path, data_handler, NULL);
     if (ret < 0)
     {
-        fprintf(stderr, "Error: Unable to parse configuration file (%d)", ret);
+        tio_error_printf("Unable to parse configuration file (%d)", ret);
         exit(EXIT_FAILURE);
     }
 
