@@ -38,6 +38,6 @@ struct config_t
 	char *map;
 };
 
-void config_file_print();
-void config_file_parse(const int argc, char *argv[]);
+void config_file_print(void);
+void config_file_parse(void);
 void config_exit(void);
