@@ -76,8 +76,8 @@ void log_open(const char *filename)
         exit(EXIT_FAILURE);
     }
 
-    // Enable full buffering
-    setvbuf(fp, file_buffer, _IOFBF, BUFSIZ);
+    // Enable line buffering
+    setvbuf(fp, file_buffer, _IOLBF, BUFSIZ);
 }
 
 bool log_strip(char c)
@@ -188,7 +188,7 @@ void log_exit(void)
 
     if (log_error)
     {
-        error_printf("Could not open log file %s (%s)", option.log_filename, strerror(errno));
+        tio_error_printf("Could not open log file %s (%s)", option.log_filename, strerror(errno));
     }
     else if (option.log)
     {
