@@ -88,11 +88,18 @@ int main(int argc, char *argv[])
     /* Initialize ANSI text formatting (colors etc.) */
     print_init_ansi_formatting();
 
+    /* Change error printing mode */
+    error_enter_session_mode();
+
     /* Print launch hints */
     tio_printf("tio v%s", VERSION);
     if (interactive_mode)
     {
-        tio_printf("Press ctrl-t q to quit");
+        tio_printf("Press ctrl-%c q to quit", option.prefix_key);
+    } else
+    {
+        tio_printf("Non-interactive mode enabled");
+        tio_printf("Press ctrl-c to quit");
     }
 
     /* Open socket */
@@ -102,7 +109,7 @@ int main(int argc, char *argv[])
     }
 
     /* Connect to tty device */
-    if ((option.no_autoconnect) || (!interactive_mode))
+    if (option.no_autoconnect)
     {
         status = tty_connect();
     }
