@@ -62,15 +62,13 @@ char *current_time(void)
         case TIMESTAMP_24HOUR_START:
             // "hh:mm:ss.sss" (24 hour format relative to start time)
             timersub(&tv_now, &tv_start, &tv);
-            tv.tv_sec -= 3600; // Why is this needed??
-            tm = localtime(&tv.tv_sec);
+            tm = gmtime(&tv.tv_sec);
             len = strftime(time_string, sizeof(time_string), "%H:%M:%S", tm);
             break;
         case TIMESTAMP_24HOUR_DELTA:
             // "hh:mm:ss.sss" (24 hour format relative to previous time stamp)
             timersub(&tv_now, &tv_previous, &tv);
-            tv.tv_sec -= 3600; // Why is this needed??
-            tm = localtime(&tv.tv_sec);
+            tm = gmtime(&tv.tv_sec);
             len = strftime(time_string, sizeof(time_string), "%H:%M:%S", tm);
             break;
         case TIMESTAMP_ISO8601:
@@ -99,6 +97,11 @@ void delay(long ms)
 {
     struct timespec ts;
 
+    if (ms <= 0)
+    {
+        return;
+    }
+
     ts.tv_sec = ms / 1000;
     ts.tv_nsec = (ms % 1000) * 1000000;
 
@@ -120,3 +123,13 @@ long string_to_long(char *string)
 
     return result;
 }
+
+int ctrl_key_code(unsigned char key)
+{
+    if ((key >= 'a') && (key <= 'z'))
+    {
+        return key & ~0x60;
+    }
+
+    return -1;
+}
