@@ -42,7 +42,7 @@ enum opt_t
     OPT_TIMESTAMP_FORMAT,
     OPT_LOG_FILE,
     OPT_LOG_STRIP,
-    OPT_DTR_PULSE_DURATION,
+    OPT_LINE_PULSE_DURATION,
 };
 
 /* Default options */
@@ -55,7 +55,13 @@ struct option_t option =
     .stopbits = 1,
     .parity = "none",
     .output_delay = 0,
+    .output_line_delay = 0,
     .dtr_pulse_duration = 100,
+    .rts_pulse_duration = 100,
+    .cts_pulse_duration = 100,
+    .dsr_pulse_duration = 100,
+    .dcd_pulse_duration = 100,
+    .ri_pulse_duration = 100,
     .no_autoconnect = false,
     .log = false,
     .log_filename = NULL,
@@ -64,8 +70,10 @@ struct option_t option =
     .timestamp = TIMESTAMP_NONE,
     .socket = NULL,
     .map = "",
-    .color = 15,
+    .color = 256, // Bold
     .hex_mode = false,
+    .prefix_code = 20, // ctrl-t
+    .prefix_key = 't',
 };
 
 void print_help(char *argv[])
@@ -75,32 +83,31 @@ void print_help(char *argv[])
     printf("Connect to tty device directly or via sub-configuration.\n");
     printf("\n");
     printf("Options:\n");
-    printf("  -b, --baudrate <bps>             Baud rate (default: 115200)\n");
-    printf("  -d, --databits 5|6|7|8           Data bits (default: 8)\n");
-    printf("  -f, --flow hard|soft|none        Flow control (default: none)\n");
-    printf("  -s, --stopbits 1|2               Stop bits (default: 1)\n");
-    printf("  -p, --parity odd|even|none       Parity (default: none)\n");
-    printf("  -o, --output-delay <ms>          Output delay (default: 0)\n");
-    printf("      --dtr-pulse-duration <ms>    DTR pulse duration (default: 100)\n");
-    printf("  -n, --no-autoconnect             Disable automatic connect\n");
-    printf("  -e, --local-echo                 Enable local echo\n");
-    printf("  -t, --timestamp                  Enable line timestamp\n");
-    printf("      --timestamp-format <format>  Set timestamp format (default: 24hour)\n");
-    printf("  -L, --list-devices               List available serial devices\n");
-    printf("  -l, --log                        Enable log to file\n");
-    printf("      --log-file <filename>        Set log filename\n");
-    printf("      --log-strip                  Strip control characters and escape sequences\n");
-    printf("  -m, --map <flags>                Map special characters\n");
-    printf("  -c, --color 0..255|none|list     Colorize tio text (default: 15)\n");
-    printf("  -S, --socket <socket>            Redirect I/O to file or network socket\n");
-    printf("  -x, --hexadecimal                Enable hexadecimal mode\n");
-    printf("  -v, --version                    Display version\n");
-    printf("  -h, --help                       Display help\n");
+    printf("  -b, --baudrate <bps>                   Baud rate (default: 115200)\n");
+    printf("  -d, --databits 5|6|7|8                 Data bits (default: 8)\n");
+    printf("  -f, --flow hard|soft|none              Flow control (default: none)\n");
+    printf("  -s, --stopbits 1|2                     Stop bits (default: 1)\n");
+    printf("  -p, --parity odd|even|none|mark|space  Parity (default: none)\n");
+    printf("  -o, --output-delay <ms>                Output character delay (default: 0)\n");
+    printf("  -O, --output-line-delay <ms>           Output line delay (default: 0)\n");
+    printf("      --line-pulse-duration <duration>   Set line pulse duration\n");
+    printf("  -n, --no-autoconnect                   Disable automatic connect\n");
+    printf("  -e, --local-echo                       Enable local echo\n");
+    printf("  -t, --timestamp                        Enable line timestamp\n");
+    printf("      --timestamp-format <format>        Set timestamp format (default: 24hour)\n");
+    printf("  -L, --list-devices                     List available serial devices\n");
+    printf("  -l, --log                              Enable log to file\n");
+    printf("      --log-file <filename>              Set log filename\n");
+    printf("      --log-strip                        Strip control characters and escape sequences\n");
+    printf("  -m, --map <flags>                      Map characters\n");
+    printf("  -c, --color 0..255|bold|none|list      Colorize tio text (default: bold)\n");
+    printf("  -S, --socket <socket>                  Redirect I/O to file or network socket\n");
+    printf("  -x, --hexadecimal                      Enable hexadecimal mode\n");
+    printf("  -v, --version                          Display version\n");
+    printf("  -h, --help                             Display help\n");
     printf("\n");
     printf("Options and sub-configurations may be set via configuration file.\n");
     printf("\n");
-    printf("In session, press ctrl-t q to quit.\n");
-    printf("\n");
     printf("See the man page for more details.\n");
 }
 
@@ -161,6 +168,63 @@ enum timestamp_t timestamp_option_parse(const char *arg)
     return timestamp;
 }
 
+void line_pulse_duration_option_parse(const char *arg)
+{
+    bool token_found = true;
+    char *token = NULL;
+    char *buffer = strdup(arg);
+
+    while (token_found == true)
+    {
+        if (token == NULL)
+        {
+            token = strtok(buffer,",");
+        }
+        else
+        {
+            token = strtok(NULL, ",");
+        }
+
+        if (token != NULL)
+        {
+            char keyname[10];
+            unsigned int value;
+            sscanf(token, "%10[^=]=%d", keyname, &value);
+
+            if (!strcmp(keyname, "DTR"))
+            {
+                option.dtr_pulse_duration = value;
+            }
+            else if (!strcmp(keyname, "RTS"))
+            {
+                option.rts_pulse_duration = value;
+            }
+            else if (!strcmp(keyname, "CTS"))
+            {
+                option.cts_pulse_duration = value;
+            }
+            else if (!strcmp(keyname, "DSR"))
+            {
+                option.dsr_pulse_duration = value;
+            }
+            else if (!strcmp(keyname, "DCD"))
+            {
+                option.dcd_pulse_duration = value;
+            }
+            else if (!strcmp(keyname, "RI"))
+            {
+                option.ri_pulse_duration = value;
+            }
+        }
+        else
+        {
+            token_found = false;
+        }
+    }
+    free(buffer);
+
+}
+
 void options_print()
 {
     tio_printf(" TTY device: %s", option.tty_device);
@@ -172,8 +236,15 @@ void options_print()
     tio_printf(" Local echo: %s", option.local_echo ? "enabled" : "disabled");
     tio_printf(" Timestamp: %s", timestamp_state_to_string(option.timestamp));
     tio_printf(" Output delay: %d", option.output_delay);
+    tio_printf(" Output line delay: %d", option.output_line_delay);
     tio_printf(" DTR pulse duration: %d", option.dtr_pulse_duration);
     tio_printf(" Auto connect: %s", option.no_autoconnect ? "disabled" : "enabled");
+    tio_printf(" Pulse duration: DTR=%d RTS=%d CTS=%d DSR=%d DCD=%d RI=%d", option.dtr_pulse_duration,
+                                                                            option.rts_pulse_duration,
+                                                                            option.cts_pulse_duration,
+                                                                            option.dsr_pulse_duration,
+                                                                            option.dcd_pulse_duration,
+                                                                            option.ri_pulse_duration);
     if (option.map[0] != 0)
         tio_printf(" Map flags: %s", option.map);
     if (option.log)
@@ -196,35 +267,36 @@ void options_parse(int argc, char *argv[])
     {
         static struct option long_options[] =
         {
-            {"baudrate",         required_argument, 0, 'b'                  },
-            {"databits",         required_argument, 0, 'd'                  },
-            {"flow",             required_argument, 0, 'f'                  },
-            {"stopbits",         required_argument, 0, 's'                  },
-            {"parity",           required_argument, 0, 'p'                  },
-            {"output-delay",     required_argument, 0, 'o'                  },
-            {"dtr-pulse-duration",   required_argument, 0, OPT_DTR_PULSE_DURATION   },
-            {"no-autoconnect",   no_argument,       0, 'n'                  },
-            {"local-echo",       no_argument,       0, 'e'                  },
-            {"timestamp",        no_argument,       0, 't'                  },
-            {"timestamp-format", required_argument, 0, OPT_TIMESTAMP_FORMAT },
-            {"list-devices",     no_argument,       0, 'L'                  },
-            {"log",              no_argument,       0, 'l'                  },
-            {"log-file",         required_argument, 0, OPT_LOG_FILE         },
-            {"log-strip",        no_argument,       0, OPT_LOG_STRIP        },
-            {"socket",           required_argument, 0, 'S'                  },
-            {"map",              required_argument, 0, 'm'                  },
-            {"color",            required_argument, 0, 'c'                  },
-            {"hexadecimal",      no_argument,       0, 'x'                  },
-            {"version",          no_argument,       0, 'v'                  },
-            {"help",             no_argument,       0, 'h'                  },
-            {0,                  0,                 0,  0                   }
+            {"baudrate",            required_argument, 0, 'b'                    },
+            {"databits",            required_argument, 0, 'd'                    },
+            {"flow",                required_argument, 0, 'f'                    },
+            {"stopbits",            required_argument, 0, 's'                    },
+            {"parity",              required_argument, 0, 'p'                    },
+            {"output-delay",        required_argument, 0, 'o'                    },
+            {"output-line-delay" ,  required_argument, 0, 'O'                    },
+            {"line-pulse-duration", required_argument, 0, OPT_LINE_PULSE_DURATION},
+            {"no-autoconnect",      no_argument,       0, 'n'                    },
+            {"local-echo",          no_argument,       0, 'e'                    },
+            {"timestamp",           no_argument,       0, 't'                    },
+            {"timestamp-format",    required_argument, 0, OPT_TIMESTAMP_FORMAT   },
+            {"list-devices",        no_argument,       0, 'L'                    },
+            {"log",                 no_argument,       0, 'l'                    },
+            {"log-file",            required_argument, 0, OPT_LOG_FILE           },
+            {"log-strip",           no_argument,       0, OPT_LOG_STRIP          },
+            {"socket",              required_argument, 0, 'S'                    },
+            {"map",                 required_argument, 0, 'm'                    },
+            {"color",               required_argument, 0, 'c'                    },
+            {"hexadecimal",         no_argument,       0, 'x'                    },
+            {"version",             no_argument,       0, 'v'                    },
+            {"help",                no_argument,       0, 'h'                    },
+            {0,                     0,                 0,  0                     }
         };
 
         /* getopt_long stores the option index here */
         int option_index = 0;
 
         /* Parse argument using getopt_long */
-        c = getopt_long(argc, argv, "b:d:f:s:p:o:netLlS:m:c:xvh", long_options, &option_index);
+        c = getopt_long(argc, argv, "b:d:f:s:p:o:O:netLlS:m:c:xvh", long_options, &option_index);
 
         /* Detect the end of the options */
         if (c == -1)
@@ -266,8 +338,12 @@ void options_parse(int argc, char *argv[])
                 option.output_delay = string_to_long(optarg);
                 break;
 
-            case OPT_DTR_PULSE_DURATION:
-                option.dtr_pulse_duration = string_to_long(optarg);
+            case 'O':
+                option.output_line_delay = string_to_long(optarg);
+                break;
+
+            case OPT_LINE_PULSE_DURATION:
+                line_pulse_duration_option_parse(optarg);
                 break;
 
             case 'n':
@@ -296,6 +372,7 @@ void options_parse(int argc, char *argv[])
                 break;
 
             case OPT_LOG_FILE:
+                option.log = true;
                 option.log_filename = optarg;
                 break;
 
@@ -322,17 +399,21 @@ void options_parse(int argc, char *argv[])
                     }
                     exit(EXIT_SUCCESS);
                 }
-
-                if (!strcmp(optarg, "none"))
+                else if (!strcmp(optarg, "none"))
                 {
-                    option.color = -1;
+                    option.color = -1; // No color
+                    break;
+                }
+                else if (!strcmp(optarg, "bold"))
+                {
+                    option.color = 256; // Bold
                     break;
                 }
 
                 option.color = string_to_long(optarg);
                 if ((option.color < 0) || (option.color > 255))
                 {
-                    printf("Error: Invalid color code\n");
+                    tio_error_printf("Invalid color code");
                     exit(EXIT_FAILURE);
                 }
                 break;
@@ -368,23 +449,23 @@ void options_parse(int argc, char *argv[])
 
     /* Assume first non-option is the tty device name */
     if (strcmp(option.tty_device, ""))
-	    optind++;
+            optind++;
     else if (optind < argc)
         option.tty_device = argv[optind++];
 
     if (strlen(option.tty_device) == 0)
     {
-        printf("Error: Missing tty device or sub-configuration name\n");
+        tio_error_printf("Missing tty device or sub-configuration name");
         exit(EXIT_FAILURE);
     }
 
     /* Print any remaining command line arguments (unknown options) */
     if (optind < argc)
     {
-        printf("Error: Unknown argument ");
+        fprintf(stderr, "Error: Unknown argument ");
         while (optind < argc)
             printf("%s ", argv[optind++]);
-        printf("\n");
+        fprintf(stderr, "\n");
         exit(EXIT_FAILURE);
     }
 }
