@@ -48,7 +48,13 @@ struct option_t
     int stopbits;
     char *parity;
     int output_delay;
-    int dtr_pulse_duration;
+    int output_line_delay;
+    unsigned int dtr_pulse_duration;
+    unsigned int rts_pulse_duration;
+    unsigned int cts_pulse_duration;
+    unsigned int dsr_pulse_duration;
+    unsigned int dcd_pulse_duration;
+    unsigned int ri_pulse_duration;
     bool no_autoconnect;
     bool log;
     bool log_strip;
@@ -59,6 +65,8 @@ struct option_t
     const char *socket;
     int color;
     bool hex_mode;
+    unsigned char prefix_code;
+    unsigned char prefix_key;
 };
 
 extern struct option_t option;
@@ -66,3 +74,5 @@ extern struct option_t option;
 void options_print();
 void options_parse(int argc, char *argv[]);
 void options_parse_final(int argc, char *argv[]);
+
+void line_pulse_duration_option_parse(const char *arg);
