@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
+#include <ctype.h>
 #include <stdlib.h>
 #include <limits.h>
 #include <sys/types.h>
@@ -48,14 +49,7 @@
 #include "log.h"
 #include "error.h"
 #include "socket.h"
-
-#ifdef HAVE_TERMIOS2
-extern int setspeed2(int fd, int baudrate);
-#endif
-
-#ifdef HAVE_IOSSIOSPEED
-extern int iossiospeed(int fd, int baudrate);
-#endif
+#include "setspeed.h"
 
 #ifdef __APPLE__
 #define PATH_SERIAL_DEVICES "/dev/"
@@ -63,6 +57,41 @@ extern int iossiospeed(int fd, int baudrate);
 #define PATH_SERIAL_DEVICES "/dev/serial/by-id/"
 #endif
 
+#ifndef CMSPAR
+#define CMSPAR   010000000000
+#endif
+
+#define KEY_0 0x30
+#define KEY_1 0x31
+#define KEY_2 0x32
+#define KEY_3 0x33
+#define KEY_4 0x34
+#define KEY_5 0x35
+#define KEY_QUESTION 0x3f
+#define KEY_B 0x62
+#define KEY_C 0x63
+#define KEY_E 0x65
+#define KEY_G 0x67
+#define KEY_H 0x68
+#define KEY_L 0x6C
+#define KEY_P 0x70
+#define KEY_Q 0x71
+#define KEY_S 0x73
+#define KEY_T 0x74
+#define KEY_U 0x55
+#define KEY_V 0x76
+#define KEY_SHIFT_L 0x4C
+
+#define NORMAL 0
+#define HEX 1
+
+enum line_mode_t
+{
+    LINE_OFF,
+    LINE_TOGGLE,
+    LINE_PULSE
+};
+
 bool interactive_mode = true;
 
 static struct termios tio, tio_old, stdout_new, stdout_old, stdin_new, stdin_old;
@@ -76,6 +105,7 @@ static bool map_i_nl_crnl = false;
 static bool map_o_cr_nl = false;
 static bool map_o_nl_crnl = false;
 static bool map_o_del_bs = false;
+static bool map_o_ltu = false;
 static char hex_chars[2];
 static unsigned char hex_char_index = 0;
 static char tty_buffer[BUFSIZ*2];
@@ -120,22 +150,23 @@ inline static unsigned char char_to_nibble(char c)
     }
 }
 
-void tty_flush(int fd)
+void tty_sync(int fd)
 {
     ssize_t count;
 
-    do
+    while (tty_buffer_count > 0)
     {
         count = write(fd, tty_buffer, tty_buffer_count);
         if (count < 0)
         {
             // Error
-            debug_printf("Write error while flushing tty buffer (%s)", strerror(errno));
+            tio_debug_printf("Write error while flushing tty buffer (%s)", strerror(errno));
             break;
         }
         tty_buffer_count -= count;
+        fsync(fd);
+        tcdrain(fd);
     }
-    while (tty_buffer_count > 0);
 
     // Reset
     tty_buffer_write_ptr = tty_buffer;
@@ -144,31 +175,52 @@ void tty_flush(int fd)
 
 ssize_t tty_write(int fd, const void *buffer, size_t count)
 {
-    ssize_t bytes_written = 0;
+    ssize_t retval = 0, bytes_written = 0;
+    size_t i;
 
-    if (option.output_delay)
+    if (map_o_ltu)
+    {
+        // Convert lower case to upper case
+        for (i = 0; i<count; i++)
+        {
+            *((unsigned char*)buffer+i) = toupper(*((unsigned char*)buffer+i));
+        }
+    }
+
+    if (option.output_delay || option.output_line_delay)
     {
         // Write byte by byte with output delay
-        for (size_t i=0; i<count; i++)
+        for (i=0; i<count; i++)
         {
-            ssize_t retval = write(fd, buffer, 1);
+            retval = write(fd, buffer, 1);
             if (retval < 0)
             {
                 // Error
-                debug_printf("Write error (%s)", strerror(errno));
+                tio_debug_printf("Write error (%s)", strerror(errno));
                 break;
             }
             bytes_written += retval;
+
+            if (option.output_line_delay && *(unsigned char*)buffer == '\r')
+            {
+                delay(option.output_line_delay);
+            }
+
             fsync(fd);
-            delay(option.output_delay);
+            tcdrain(fd);
+
+            if (option.output_delay)
+            {
+                delay(option.output_delay);
+            }
         }
     }
     else
     {
-        // Flush tty buffer if too full
+        // Force write of tty buffer if too full
         if ((tty_buffer_count + count) > BUFSIZ)
         {
-            tty_flush(fd);
+            tty_sync(fd);
         }
 
         // Copy bytes to tty write buffer
@@ -195,7 +247,7 @@ static void output_hex(char c)
         ssize_t status = tty_write(fd, &hex_value, 1);
         if (status < 0)
         {
-            warning_printf("Could not write to tty device");
+            tio_warning_printf("Could not write to tty device");
         }
         else
         {
@@ -204,28 +256,67 @@ static void output_hex(char c)
     }
 }
 
-static void toggle_line(const char *line_name, int mask)
+static void toggle_line(const char *line_name, int mask, enum line_mode_t line_mode)
 {
     int state;
 
-    if (ioctl(fd, TIOCMGET, &state) < 0)
+    if (line_mode == LINE_TOGGLE)
     {
-        warning_printf("Could not get line state (%s)", strerror(errno));
-    }
-    else
-    {
-        if (state & mask)
+        // Toggle line
+        if (ioctl(fd, TIOCMGET, &state) < 0)
         {
-            state &= ~mask;
-            tio_printf("set %s to LOW", line_name);
+            tio_warning_printf("Could not get line state (%s)", strerror(errno));
         }
         else
         {
-            state |= mask;
-            tio_printf("set %s to HIGH", line_name);
+            if (state & mask)
+            {
+                state &= ~mask;
+                tio_printf("Setting %s to LOW", line_name);
+            }
+            else
+            {
+                state |= mask;
+                tio_printf("Setting %s to HIGH", line_name);
+            }
+            if (ioctl(fd, TIOCMSET, &state) < 0)
+                tio_warning_printf("Could not set line state (%s)", strerror(errno));
         }
-        if (ioctl(fd, TIOCMSET, &state) < 0)
-            warning_printf("Could not set line state (%s)", strerror(errno));
+    } else if (line_mode == LINE_PULSE)
+    {
+        int duration = 0;
+        // Pulse line
+        toggle_line(line_name, mask, LINE_TOGGLE);
+        switch (mask)
+        {
+            case TIOCM_DTR:
+                duration = option.dtr_pulse_duration;
+                break;
+            case TIOCM_RTS:
+                duration = option.rts_pulse_duration;
+                break;
+            case TIOCM_CTS:
+                duration = option.cts_pulse_duration;
+                break;
+            case TIOCM_DSR:
+                duration = option.dsr_pulse_duration;
+                break;
+            case TIOCM_CD:
+                duration = option.dcd_pulse_duration;
+                break;
+            case TIOCM_RI:
+                duration = option.ri_pulse_duration;
+                break;
+            default:
+                duration = 0;
+                break;
+        }
+        if (duration > 0)
+        {
+            tio_printf("Waiting %d ms", duration);
+            delay(duration);
+        }
+        toggle_line(line_name, mask, LINE_TOGGLE);
     }
 }
 
@@ -234,16 +325,55 @@ void handle_command_sequence(char input_char, char previous_char, char *output_c
     char unused_char;
     bool unused_bool;
     int state;
+    static enum line_mode_t line_mode = LINE_OFF;
 
     /* Ignore unused arguments */
     if (output_char == NULL)
+    {
         output_char = &unused_char;
+    }
 
     if (forward == NULL)
+    {
         forward = &unused_bool;
+    }
+
+    if (line_mode)
+    {
+        // Handle line toggle number action
+        *forward = false;
+        switch (input_char)
+        {
+            case KEY_0:
+                toggle_line("DTR", TIOCM_DTR, line_mode);
+                break;
+            case KEY_1:
+                toggle_line("RTS", TIOCM_RTS, line_mode);
+                break;
+            case KEY_2:
+                toggle_line("CTS", TIOCM_CTS, line_mode);
+                break;
+            case KEY_3:
+                toggle_line("DSR", TIOCM_DSR, line_mode);
+                break;
+            case KEY_4:
+                toggle_line("DCD", TIOCM_CD, line_mode);
+                break;
+            case KEY_5:
+                toggle_line("RI", TIOCM_RI, line_mode);
+                break;
+            default:
+                tio_warning_printf("Invalid line number");
+                break;
+        }
+
+        line_mode = LINE_OFF;
+
+        return;
+    }
 
     /* Handle escape key commands */
-    if (previous_char == KEY_CTRL_T)
+    if (previous_char == option.prefix_code)
     {
         /* Do not forward input char to output by default */
         *forward = false;
@@ -252,27 +382,26 @@ void handle_command_sequence(char input_char, char previous_char, char *output_c
         {
             case KEY_QUESTION:
                 tio_printf("Key commands:");
-                tio_printf(" ctrl-t ?   List available key commands");
-                tio_printf(" ctrl-t b   Send break");
-                tio_printf(" ctrl-t c   Show configuration");
-                tio_printf(" ctrl-t d   Toggle DTR line");
-                tio_printf(" ctrl-t D   Pulse DTR line");
-                tio_printf(" ctrl-t e   Toggle local echo mode");
-                tio_printf(" ctrl-t h   Toggle hexadecimal mode");
-                tio_printf(" ctrl-t l   Clear screen");
-                tio_printf(" ctrl-t L   Show line states");
-                tio_printf(" ctrl-t q   Quit");
-                tio_printf(" ctrl-t r   Toggle RTS line");
-                tio_printf(" ctrl-t s   Show statistics");
-                tio_printf(" ctrl-t t   Send ctrl-t key code");
-                tio_printf(" ctrl-t T   Toggle line timestamp mode");
-                tio_printf(" ctrl-t v   Show version");
+                tio_printf(" ctrl-%c ?   List available key commands", option.prefix_key);
+                tio_printf(" ctrl-%c b   Send break", option.prefix_key);
+                tio_printf(" ctrl-%c c   Show configuration", option.prefix_key);
+                tio_printf(" ctrl-%c e   Toggle local echo mode", option.prefix_key);
+                tio_printf(" ctrl-%c g   Toggle serial port line", option.prefix_key);
+                tio_printf(" ctrl-%c h   Toggle hexadecimal mode", option.prefix_key);
+                tio_printf(" ctrl-%c l   Clear screen", option.prefix_key);
+                tio_printf(" ctrl-%c L   Show line states", option.prefix_key);
+                tio_printf(" ctrl-%c p   Pulse serial port line", option.prefix_key);
+                tio_printf(" ctrl-%c q   Quit", option.prefix_key);
+                tio_printf(" ctrl-%c s   Show statistics", option.prefix_key);
+                tio_printf(" ctrl-%c t   Toggle line timestamp mode", option.prefix_key);
+                tio_printf(" ctrl-%c U   Toggle conversion to uppercase on output", option.prefix_key);
+                tio_printf(" ctrl-%c v   Show version", option.prefix_key);
                 break;
 
             case KEY_SHIFT_L:
                 if (ioctl(fd, TIOCMGET, &state) < 0)
                 {
-                    warning_printf("Could not get line state (%s)", strerror(errno));
+                    tio_warning_printf("Could not get line state (%s)", strerror(errno));
                     break;
                 }
                 tio_printf("Line states:");
@@ -283,18 +412,29 @@ void handle_command_sequence(char input_char, char previous_char, char *output_c
                 tio_printf(" DCD: %s", (state & TIOCM_CD) ? "HIGH" : "LOW");
                 tio_printf(" RI : %s", (state & TIOCM_RI) ? "HIGH" : "LOW");
                 break;
-            case KEY_D:
-                toggle_line("DTR", TIOCM_DTR);
+
+            case KEY_G:
+                tio_printf("Please enter which serial line number to toggle:");
+                tio_printf(" DTR (0)");
+                tio_printf(" RTS (1)");
+                tio_printf(" CTS (2)");
+                tio_printf(" DSR (3)");
+                tio_printf(" DCD (4)");
+                tio_printf(" RI  (5)");
+                // Process next input character as part of the line toggle step
+                line_mode = LINE_TOGGLE;
                 break;
 
-            case KEY_SHIFT_D:
-                toggle_line("DTR", TIOCM_DTR);
-                delay(option.dtr_pulse_duration);
-                toggle_line("DTR", TIOCM_DTR);
-                break;
-
-            case KEY_R:
-                toggle_line("RTS", TIOCM_RTS);
+            case KEY_P:
+                tio_printf("Please enter which serial line number to pulse:");
+                tio_printf(" DTR (0)");
+                tio_printf(" RTS (1)");
+                tio_printf(" CTS (2)");
+                tio_printf(" DSR (3)");
+                tio_printf(" DCD (4)");
+                tio_printf(" RI  (5)");
+                // Process next input character as part of the line pulse step
+                line_mode = LINE_PULSE;
                 break;
 
             case KEY_B:
@@ -345,12 +485,6 @@ void handle_command_sequence(char input_char, char previous_char, char *output_c
                 break;
 
             case KEY_T:
-                /* Send ctrl-t key code upon ctrl-t t sequence */
-                *output_char = KEY_CTRL_T;
-                *forward = true;
-                break;
-
-            case KEY_SHIFT_T:
                 option.timestamp += 1;
                 switch (option.timestamp)
                 {
@@ -375,6 +509,10 @@ void handle_command_sequence(char input_char, char previous_char, char *output_c
                 }
                 break;
 
+            case KEY_U:
+                map_o_ltu = !map_o_ltu;
+                break;
+
             case KEY_V:
                 tio_printf("tio v%s", VERSION);
                 break;
@@ -398,7 +536,7 @@ void stdin_configure(void)
     /* Save current stdin settings */
     if (tcgetattr(STDIN_FILENO, &stdin_old) < 0)
     {
-        error_printf("Saving current stdin settings failed");
+        tio_error_printf("Saving current stdin settings failed");
         exit(EXIT_FAILURE);
     }
 
@@ -416,7 +554,7 @@ void stdin_configure(void)
     status = tcsetattr(STDIN_FILENO, TCSANOW, &stdin_new);
     if (status == -1)
     {
-        error_printf("Could not apply new stdin settings (%s)", strerror(errno));
+        tio_error_printf("Could not apply new stdin settings (%s)", strerror(errno));
         exit(EXIT_FAILURE);
     }
 
@@ -440,7 +578,7 @@ void stdout_configure(void)
     /* Save current stdout settings */
     if (tcgetattr(STDOUT_FILENO, &stdout_old) < 0)
     {
-        error_printf("Saving current stdio settings failed");
+        tio_error_printf("Saving current stdio settings failed");
         exit(EXIT_FAILURE);
     }
 
@@ -450,6 +588,12 @@ void stdout_configure(void)
     /* Reconfigure stdout (RAW configuration) */
     cfmakeraw(&stdout_new);
 
+    /* Allow ^C / SIGINT (to allow termination when piping to tio) */
+    if (!interactive_mode)
+    {
+        stdout_new.c_lflag |= ISIG;
+    }
+
     /* Control characters */
     stdout_new.c_cc[VTIME] = 0; /* Inter-character timer unused */
     stdout_new.c_cc[VMIN]  = 1; /* Blocking read until 1 character received */
@@ -458,7 +602,7 @@ void stdout_configure(void)
     status = tcsetattr(STDOUT_FILENO, TCSANOW, &stdout_new);
     if (status == -1)
     {
-        error_printf("Could not apply new stdout settings (%s)", strerror(errno));
+        tio_error_printf("Could not apply new stdout settings (%s)", strerror(errno));
         exit(EXIT_FAILURE);
     }
 
@@ -500,7 +644,7 @@ void tty_configure(void)
             standard_baudrate = false;
             break;
 #else
-            error_printf("Invalid baud rate");
+            tio_error_printf("Invalid baud rate");
             exit(EXIT_FAILURE);
 #endif
     }
@@ -511,7 +655,7 @@ void tty_configure(void)
         status = cfsetispeed(&tio, baudrate);
         if (status == -1)
         {
-            error_printf("Could not configure input speed (%s)", strerror(errno));
+            tio_error_printf("Could not configure input speed (%s)", strerror(errno));
             exit(EXIT_FAILURE);
         }
 
@@ -519,7 +663,7 @@ void tty_configure(void)
         status = cfsetospeed(&tio, baudrate);
         if (status == -1)
         {
-            error_printf("Could not configure output speed (%s)", strerror(errno));
+            tio_error_printf("Could not configure output speed (%s)", strerror(errno));
             exit(EXIT_FAILURE);
         }
     }
@@ -541,7 +685,7 @@ void tty_configure(void)
             tio.c_cflag |= CS8;
             break;
         default:
-            error_printf("Invalid data bits");
+            tio_error_printf("Invalid data bits");
             exit(EXIT_FAILURE);
     }
 
@@ -563,7 +707,7 @@ void tty_configure(void)
     }
     else
     {
-        error_printf("Invalid flow control");
+        tio_error_printf("Invalid flow control");
         exit(EXIT_FAILURE);
     }
 
@@ -577,7 +721,7 @@ void tty_configure(void)
             tio.c_cflag |= CSTOPB;
             break;
         default:
-            error_printf("Invalid stop bits");
+            tio_error_printf("Invalid stop bits");
             exit(EXIT_FAILURE);
     }
 
@@ -596,9 +740,21 @@ void tty_configure(void)
     {
         tio.c_cflag &= ~PARENB;
     }
+    else if ( strcmp("mark", option.parity) == 0)
+    {
+        tio.c_cflag |= PARENB;
+        tio.c_cflag |= PARODD;
+        tio.c_cflag |= CMSPAR;
+    }
+    else if ( strcmp("space", option.parity) == 0)
+    {
+        tio.c_cflag |= PARENB;
+        tio.c_cflag &= ~PARODD;
+        tio.c_cflag |= CMSPAR;
+    }
     else
     {
-        error_printf("Invalid parity");
+        tio_error_printf("Invalid parity");
         exit(EXIT_FAILURE);
     }
 
@@ -654,6 +810,10 @@ void tty_configure(void)
             {
                 map_o_nl_crnl = true;
             }
+            else if (strcmp(token, "OLTU") == 0)
+            {
+                map_o_ltu = true;
+            }
             else
             {
                 printf("Error: Unknown mapping flag %s\n", token);
@@ -681,51 +841,56 @@ void tty_wait_for_device(void)
     /* Loop until device pops up */
     while (true)
     {
-        if (first)
+        if (interactive_mode)
         {
-            /* Don't wait first time */
-            tv.tv_sec = 0;
-            tv.tv_usec = 1;
-            first = false;
-        }
-        else
-        {
-            /* Wait up to 1 second */
-            tv.tv_sec = 1;
-            tv.tv_usec = 0;
-        }
-
-        FD_ZERO(&rdfs);
-        FD_SET(STDIN_FILENO, &rdfs);
-        maxfd = MAX(STDIN_FILENO, socket_add_fds(&rdfs, false));
-
-        /* Block until input becomes available or timeout */
-        status = select(maxfd + 1, &rdfs, NULL, NULL, &tv);
-        if (status > 0)
-        {
-            if (FD_ISSET(STDIN_FILENO, &rdfs))
+            /* In interactive mode, while waiting for tty device, we need to
+             * read from stdin to react on input key commands. */
+            if (first)
             {
-                /* Input from stdin ready */
+                /* Don't wait first time */
+                tv.tv_sec = 0;
+                tv.tv_usec = 1;
+                first = false;
+            }
+            else
+            {
+                /* Wait up to 1 second for input */
+                tv.tv_sec = 1;
+                tv.tv_usec = 0;
+            }
 
-                /* Read one character */
-                status = read(STDIN_FILENO, &input_char, 1);
-                if (status <= 0)
+            FD_ZERO(&rdfs);
+            FD_SET(STDIN_FILENO, &rdfs);
+            maxfd = MAX(STDIN_FILENO, socket_add_fds(&rdfs, false));
+
+            /* Block until input becomes available or timeout */
+            status = select(maxfd + 1, &rdfs, NULL, NULL, &tv);
+            if (status > 0)
+            {
+                if (FD_ISSET(STDIN_FILENO, &rdfs))
                 {
-                    error_printf("Could not read from stdin");
-                    exit(EXIT_FAILURE);
-                }
+                    /* Input from stdin ready */
+
+                    /* Read one character */
+                    status = read(STDIN_FILENO, &input_char, 1);
+                    if (status <= 0)
+                    {
+                        tio_error_printf("Could not read from stdin");
+                        exit(EXIT_FAILURE);
+                    }
 
-                /* Handle commands */
-                handle_command_sequence(input_char, previous_char, NULL, NULL);
+                    /* Handle commands */
+                    handle_command_sequence(input_char, previous_char, NULL, NULL);
 
-                previous_char = input_char;
+                    previous_char = input_char;
+                }
+                socket_handle_input(&rdfs, NULL);
+            }
+            else if (status == -1)
+            {
+                tio_error_printf("select() failed (%s)", strerror(errno));
+                exit(EXIT_FAILURE);
             }
-            socket_handle_input(&rdfs, NULL);
-        }
-        else if (status == -1)
-        {
-            error_printf("select() failed (%s)", strerror(errno));
-            exit(EXIT_FAILURE);
         }
 
         /* Test for accessible device file */
@@ -737,10 +902,18 @@ void tty_wait_for_device(void)
         }
         else if (last_errno != errno)
         {
-            warning_printf("Could not open tty device (%s)", strerror(errno));
+            tio_warning_printf("Could not open tty device (%s)", strerror(errno));
             tio_printf("Waiting for tty device..");
             last_errno = errno;
         }
+
+        if (!interactive_mode)
+        {
+            /* In non-interactive mode we do not need to handle input key
+             * commands so we simply sleep 1 second between checking for
+             * presence of tty device */
+            sleep(1);
+        }
     }
 }
 
@@ -789,7 +962,7 @@ void forward_to_tty(int fd, char output_char)
         status = tty_write(fd, crlf, 2);
         if (status < 0)
         {
-            warning_printf("Could not write to tty device");
+            tio_warning_printf("Could not write to tty device");
         }
 
         tx_total += 2;
@@ -807,7 +980,7 @@ void forward_to_tty(int fd, char output_char)
             status = tty_write(fd, &output_char, 1);
             if (status < 0)
             {
-                warning_printf("Could not write to tty device");
+                tio_warning_printf("Could not write to tty device");
             }
 
             /* Update transmit statistics */
@@ -832,14 +1005,14 @@ int tty_connect(void)
     fd = open(option.tty_device, O_RDWR | O_NOCTTY | O_NONBLOCK);
     if (fd < 0)
     {
-        error_printf_silent("Could not open tty device (%s)", strerror(errno));
+        tio_error_printf_silent("Could not open tty device (%s)", strerror(errno));
         goto error_open;
     }
 
     /* Make sure device is of tty type */
     if (!isatty(fd))
     {
-        error_printf("Not a tty device");
+        tio_error_printf("Not a tty device");
         exit(EXIT_FAILURE);;
     }
 
@@ -847,7 +1020,7 @@ int tty_connect(void)
     status = flock(fd, LOCK_EX | LOCK_NB);
     if ((status == -1) && (errno == EWOULDBLOCK))
     {
-        error_printf("Device file is locked by another process");
+        tio_error_printf("Device file is locked by another process");
         exit(EXIT_FAILURE);
     }
 
@@ -879,13 +1052,14 @@ int tty_connect(void)
     /* Save current port settings */
     if (tcgetattr(fd, &tio_old) < 0)
     {
+        tio_error_printf_silent("Could not get port settings (%s)", strerror(errno));
         goto error_tcgetattr;
     }
 
 #ifdef HAVE_IOSSIOSPEED
     if (!standard_baudrate)
     {
-        /* OS X wants these fields left alone. We'll set baudrate with iossiospeed below. */
+        /* OS X wants these fields left alone before setting arbitrary baud rate */
         tio.c_ispeed = tio_old.c_ispeed;
         tio.c_ospeed = tio_old.c_ospeed;
     }
@@ -902,31 +1076,19 @@ int tty_connect(void)
     status = tcsetattr(fd, TCSANOW, &tio);
     if (status == -1)
     {
-        error_printf_silent("Could not apply port settings (%s)", strerror(errno));
+        tio_error_printf_silent("Could not apply port settings (%s)", strerror(errno));
         goto error_tcsetattr;
     }
 
-#ifdef HAVE_TERMIOS2
+    /* Set arbitrary baudrate (only works on supported platforms) */
     if (!standard_baudrate)
     {
-        if (setspeed2(fd, option.baudrate) != 0)
+        if (setspeed(fd, option.baudrate) != 0)
         {
-            error_printf_silent("Could not set baudrate speed (%s)", strerror(errno));
+            tio_error_printf_silent("Could not set baudrate speed (%s)", strerror(errno));
             goto error_setspeed;
         }
     }
-#endif
-
-#ifdef HAVE_IOSSIOSPEED
-    if (!standard_baudrate)
-    {
-        if (iossiospeed(fd, option.baudrate) != 0)
-        {
-            error_printf_silent("Could not set baudrate speed (%s)", strerror(errno));
-            goto error_setspeed;
-        }
-    }
-#endif
 
     /* Input loop */
     while (true)
@@ -949,7 +1111,7 @@ int tty_connect(void)
                 if (bytes_read <= 0)
                 {
                     /* Error reading - device is likely unplugged */
-                    error_printf_silent("Could not read from tty device");
+                    tio_error_printf_silent("Could not read from tty device");
                     goto error_read;
                 }
 
@@ -1012,11 +1174,17 @@ int tty_connect(void)
             {
                 /* Input from stdin ready */
                 ssize_t bytes_read = read(STDIN_FILENO, input_buffer, BUFSIZ);
-                if (bytes_read <= 0)
+                if (bytes_read < 0)
                 {
-                    error_printf_silent("Could not read from stdin");
+                    tio_error_printf_silent("Could not read from stdin (%s)", strerror(errno));
                     goto error_read;
                 }
+                else if (bytes_read == 0)
+                {
+                    /* Reached EOF (when piping to stdin) */
+                    tty_sync(fd);
+                    exit(EXIT_SUCCESS);
+                }
 
                 /* Process input byte by byte */
                 for (int i=0; i<bytes_read; i++)
@@ -1029,8 +1197,8 @@ int tty_connect(void)
 
                     if (interactive_mode)
                     {
-                        /* Do not forward ctrl-t key */
-                        if (input_char == KEY_CTRL_T)
+                        /* Do not forward prefix key */
+                        if (input_char == option.prefix_code)
                         {
                             forward = false;
                         }
@@ -1041,11 +1209,11 @@ int tty_connect(void)
                         /* Save previous key */
                         previous_char = input_char;
 
-                        if (print_mode == HEX)
+                        if ((print_mode == HEX) && (forward))
                         {
                             if (!is_valid_hex(input_char))
                             {
-                                warning_printf("Invalid hex character: '%d' (0x%02x)", input_char, input_char);
+                                tio_warning_printf("Invalid hex character: '%d' (0x%02x)", input_char, input_char);
                                 forward = false;
                             }
                         }
@@ -1057,7 +1225,7 @@ int tty_connect(void)
                     }
                 }
 
-                tty_flush(fd);
+                tty_sync(fd);
             }
             else
             {
@@ -1068,21 +1236,19 @@ int tty_connect(void)
                     forward_to_tty(fd, output_char);
                 }
 
-                tty_flush(fd);
+                tty_sync(fd);
             }
         }
         else if (status == -1)
         {
-            error_printf("Error: select() failed (%s)", strerror(errno));
+            tio_error_printf("select() failed (%s)", strerror(errno));
             exit(EXIT_FAILURE);
         }
     }
 
     return TIO_SUCCESS;
 
-#if defined (HAVE_TERMIOS2) || defined (HAVE_IOSSIOSPEED)
 error_setspeed:
-#endif
 error_tcsetattr:
 error_tcgetattr:
 error_read:
